/***************************************************************************
 *   Copyright (C) 2006-2012 by Julien Lemoine, Simon Viennot              *
 *   Web site: http://kitsune.tuxfamily.org                                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "calculation.h"
#include "selection.h"

//Initialization of a void calcul, by default it is supposed additive
calcul::calcul() {
	isTile=false;
	isAdditive=true;
	sign=true;		//positive by default
	value=0;	//0 by default
}

//Initialization of a calcul from a single tile.
calcul::calcul(const tile &tile1) {
	isTile=true;
	pl=tile1;
	value=tile1;
	
	isAdditive=true;	//additive by default
	sign=true;		//positive by default
}

//initialization of the static variable (by deafult, we are on the top level)
//used to put parenthesis only on the inferior levels
bool calcul::topLevel=true;

//Display a calcul recursively
//calculs are displayed in reverse order of the memory storage
//(calculs are stored in increasing order for the < operator defined below, but displayed in decreasing order for <).
string calcul::display() const {
	string result;
	
	if (isTile){
		stringstream ss;
		ss << (int) pl;
		ss >> result;
	}
	else{
		list<calcul>::const_reverse_iterator it;
		string Plus=string("x"), Minus=string("/"), Parenth_debut, Parenth_fin;
		
		if (isAdditive){
			Plus="+"; Minus="-";
			if (!calcul::topLevel) {
				Parenth_debut="("; Parenth_fin=")";
			}
		}
		
		bool saveTopLevel=calcul::topLevel;
		calcul::topLevel=false;			//topLevel is set to false
		result.append(Parenth_debut);
		
		it=sub_calculs.rbegin();
		result.append(it->display());				//display the first additive calcul
		it++;
		
		while(it!=sub_calculs.rend()) {
			if (it->sign==true) {
				result.append(Plus);
			}
			else {
				result.append(Minus);
			}
			result.append(it->display());
			it++;
		}
		
		result.append(Parenth_fin);
		calcul::topLevel=saveTopLevel;			//put topLevel back to its value
	}
	return result;
}

//recursive equality operator
bool calcul::operator==(const calcul& a) const{
	if (value!=a.value) return false;
	if(isTile!=a.isTile || isAdditive!=a.isAdditive || sign!=a.sign) return false;
	if(isTile) return pl==a.pl;
	
	//calculs are equal if all their sub_calculs are equal
	//list<calcul>::operator== calls calcul::operator==, hence the recursivity
	return sub_calculs==a.sub_calculs;
}

//Recursive comparison operator on the "calcul"
//Comparison rules :
// 0.negative calcul < positive calcul
// 1.small value < big value
// 2.tile < complex calcul
// 3.additive calcul < multiplicative calcul
// 4.short calcul < long calcul
// 5.recursivity on the list of sub-calculs
bool calcul::operator<(const calcul& a) const{
	if (sign != a.sign)             return !sign;				//rule 0
	if (value != a.value)           return value<a.value;		//rule 1
	if (isTile != a.isTile)         return isTile;				//rule 2
	if (isAdditive != a.isAdditive) return isAdditive;			//rule 3
	
	int nb  =  sub_calculs.size();
	int nb_a=a.sub_calculs.size();
	if(nb!=nb_a)                   return nb<nb_a;						//rule 4
	if(sub_calculs!=a.sub_calculs) return sub_calculs<a.sub_calculs;	//rule 5 (recursivity)
	return false;					//in case of equality
}

//addition operator, which should never be called on a void calcul
calcul calcul::operator+(const calcul& a) const {
	calcul result=reunion(a, true, false);			//additiveType==true and opposition==false
	result.value=value+a.value;
	return result;
}

//substraction operator, we suppose that this.value>a.value
calcul calcul::operator-(const calcul& a) const {
	calcul result=reunion(a, true, true);				//additiveType==true and opposition==true
	result.value=value-a.value;
	return result;
}

//multiplication operator
calcul calcul::operator*(const calcul& a) const {
	calcul result=reunion(a, false, false);			//additiveType==false and opposition==false
	result.value=value*a.value;
	return result;
}

//division operator, we suppose that a.value divides this.value...
calcul calcul::operator/(const calcul& a) const {
	calcul result=reunion(a, false, true);			//additiveType==false and opposition==true
	result.value=value/a.value;
	return result;
}

//reverse the signs of the sub-calculs
void calcul::reverseSign() {
	list<calcul>::iterator Ci; 			//calcul i
	
	for(Ci=sub_calculs.begin(); Ci!=sub_calculs.end(); Ci++){
		Ci->sign=!(Ci->sign);
	}
}

//unique function to do all the +, -, *, / operations : we have result=(*this)op(a) with op = +, -, *, /
//additiveType and opposition describe the kind of operation
// "+" : additiveType=true opposition=false
// "-" : additiveType=true opposition=true
// "*" : additiveType=false opposition=false
// "/" : additiveType=false opposition=true
calcul calcul::reunion(const calcul& a, bool additiveType, bool opposition) const {
	calcul result;
	result.isAdditive=additiveType;
	
	//if a is not a calcul of the same kind as the operation, then a is a sub-calcul of the result
	//otherwise it is the subcalculs of a that are sub-calculs of the result
	if(a.isTile || a.isAdditive!=additiveType) {
		result.sub_calculs.push_back(a);
	} else {
		result.sub_calculs=a.sub_calculs;
	}
	
	//if the operation is an opposition, we reverse the sign of the sub-calcul
	if(opposition==true) result.reverseSign();
	
	//add *this to the result, just in the same way as a
	if(isTile || isAdditive!=additiveType) {
		result.sub_calculs.push_back(*this);
	} else {
		result.sub_calculs.insert(result.sub_calculs.end(), sub_calculs.begin(), sub_calculs.end());
	}
	
	result.sub_calculs.sort(); //sort the sub_calculs
	return result;
}

//--------------------------------------------------------------
//Functions to analyze the calculs
//--------------------------------------------------------------

//return the list of sub_calculs in a broad meaning, of depth 1,
//and size >=2 because those of size 1 have already been checked as canonical
list<calcul> calcul::sub_calculs_list() const{
	list<calcul> result;
	if(isTile){
		return result;
	}

	int nb=sub_calculs.size();
	calcul calc_temp;
	targetType value_calc_temp;
	calc_temp.isAdditive=isAdditive;
	
	//powers of 2 indicating which calculs we consider
	int deux_puiss_pos=1;
	int i;
	for(i=0;i<nb;i++){
		deux_puiss_pos=deux_puiss_pos*2;
	}
	
	list<calcul>::const_iterator Ck;
	
	for(i=0;i<deux_puiss_pos;i++){
		int k=i;
		
		if (isAdditive) {
			value_calc_temp=0;
		}
		else {
			value_calc_temp=1;
		}
		
		calc_temp.sub_calculs.clear();
		targetType value_calc_div=1;
		
		for(Ck=sub_calculs.begin(); Ck!=sub_calculs.end(); Ck++){
			if(k%2==1){
				calc_temp.sub_calculs.push_back(*Ck);
				if(isAdditive){
					if (Ck->sign==true) {
						value_calc_temp += Ck->value;
					}
					else {
						value_calc_temp -= Ck->value;
					}
				}
				else{
					if (Ck->sign==true) {
						value_calc_temp *= Ck->value;
					}
					else {
						value_calc_div *= Ck->value;
					}
				}
			}
			
			k=k/2;
		}
		
		//consider sub-calculs with size verifying : 2 <= size < size_calcul
		bool valable=true;
		int nb_calculs=calc_temp.sub_calculs.size();
		
		if (nb_calculs>1 && nb_calculs<nb){
		
			if(isAdditive && value_calc_temp<0){
				value_calc_temp=-value_calc_temp;
				calc_temp.reverseSign();
			}
			
			if(!isAdditive){
				if(value_calc_temp%value_calc_div==0){
					value_calc_temp /= value_calc_div;
				}
				else {
					if(value_calc_div%value_calc_temp==0){
						value_calc_temp = value_calc_div/value_calc_temp;
						calc_temp.reverseSign();
					}
					else{
						valable=false;
					}
				}
			}
			
			if(valable){
				calc_temp.sub_calculs.sort();
				calc_temp.value=value_calc_temp;
				result.push_back(calc_temp);
			}
		}
	}
	return result;
}

//Compute the list of targets obtained by re-organizing the children
//If "useAllChildren" is true, we can use all the children in a reorganization. Otherwise, at most one less.
list<targetType> calcul::children_reachableTarget(bool useAllChildren) const{
	list<targetType> result;
	
	list<calcul>::const_iterator Vi;			//value i
	list<calcul>::const_iterator Vdeb=sub_calculs.begin();	//beginning of the list of values (optimization)
	
	int i, i_temp, j;
	int taille_liste=sub_calculs.size();
	
	int trois_puissance=1;		//no power function in c++ ?
	for(i=1;i<=taille_liste;i++){
		trois_puissance=trois_puissance*3;
	}
	
	bool contient_zero;
	targetType somme, prod, div;
	
	for(i=1;i<trois_puissance;i++){		//tricky code with powers of three
		contient_zero=false;
		somme=0; prod=1; div=1;
		i_temp=i;
		
		for (j=0, Vi=Vdeb; j<taille_liste; j++, Vi++){
			switch(i_temp%3){
				case 0:contient_zero=true; break;
				case 1: if(isAdditive) {
						somme+=Vi->value;
					} else {
						prod*=Vi->value;
					}
					break;
				case 2: if(isAdditive) {
						somme-=Vi->value;
					} else {
						div*=Vi->value;
					}
					break;
			}
			i_temp=i_temp/3;
		}
		
		if(isAdditive && (contient_zero==true || useAllChildren==true) && somme>0){
			result.push_back(somme);
		}

		if(!isAdditive && (contient_zero==true || useAllChildren==true) && prod%div==0){
			result.push_back(prod/div);
		}
	}
	
	return result;
}

//recursive function to compute the list of targets reachable from a strict subset of the tiles used in the calcul
//Sould be used with "useAllChildren" set to false
list<targetType> calcul::subSet_reachableTarget(bool useAllChildren) const{
	list<targetType> result=children_reachableTarget(useAllChildren);
	list<calcul>::const_iterator Ci;
	
	for(Ci=sub_calculs.begin();Ci!=sub_calculs.end();Ci++){
		if(!(Ci->isTile)){
			list<targetType> popo=(Ci->subSet_reachableTarget(true));
			result.splice(result.end(),popo);
		}
	}
	
	return result;
}

//returns the list of values of sub-calculs of depth 1 in a broad meaning, with size >=1 and < size_calcul (and so different from the calcul)
//the result is negative if the sub-calcul is in the reverse order :
//for example, for 10+2-8, the sub-calcul (8-2) gives the sub-target -6.
//for 100*3/6, the sub-calcul gives the sub-target -2.
//Function derived from sub_calculs_list, maybe could be merged ?
list<targetType> calcul::subTargetList() const{
	list<targetType> result;
	if(isTile){
		return result;
	}

	int nb=sub_calculs.size();
	calcul calc_temp;
	targetType value_calc_temp;
	calc_temp.isAdditive=isAdditive;
	
	//powers of 2 indicating which calculs we consider
	int deux_puiss_pos=1;
	int i;
	for(i=0;i<nb;i++){
		deux_puiss_pos=deux_puiss_pos*2;
	}
	
	list<calcul>::const_iterator Ck;
	
	for(i=0;i<deux_puiss_pos;i++){
		int k=i;
		
		if (isAdditive) {
			value_calc_temp=0;
		}
		else {
			value_calc_temp=1;
		}
		
		calc_temp.sub_calculs.clear();
		targetType value_calc_div=1;
		
		for(Ck=sub_calculs.begin(); Ck!=sub_calculs.end(); Ck++){
			if(k%2==1){
				calc_temp.sub_calculs.push_back(*Ck);
				if(isAdditive){
					if (Ck->sign==true) {
						value_calc_temp += Ck->value;
					}
					else {
						value_calc_temp -= Ck->value;
					}
				}
				else{
					if (Ck->sign==true) {
						value_calc_temp *= Ck->value;
					}
					else {
						value_calc_div *= Ck->value;
					}
				}
			}
			
			k=k/2;
		}
		
		//consider sub-calculs with size>=1 and < size_calcul
		bool valable=true;
		int nb_calculs=calc_temp.sub_calculs.size();
		
		if (nb_calculs>=1 && nb_calculs<nb){
			
			bool sign_result=true; 		//true for a positive result and false for a negative one
			
			if(isAdditive && value_calc_temp<0){
				sign_result=false;
				//value_calc_temp=-value_calc_temp;
				//calc_temp.sub_calculs=calc_temp.reverseSign();
			}
			
			if(!isAdditive){
				if(value_calc_temp%value_calc_div==0){
					value_calc_temp /= value_calc_div;
				}
				else {
					if(value_calc_div%value_calc_temp==0){
						sign_result=false;
						value_calc_temp = -value_calc_div/value_calc_temp;
						//calc_temp.sub_calculs=calc_temp.reverseSign();
					}
					else{
						valable=false;
					}
				}
			}
			
			if(valable){
				//calc_temp.sub_calculs.sort();
				result.push_back(value_calc_temp);
			}
		}
	}
	return result;
}

//return the list of tiles used by a calcul
selection calcul::tileList() const{
	selection result;
	if(isTile){
		result.tab[pl]=1;
	} else{
		list<calcul>::const_iterator Ci;
		for(Ci=sub_calculs.begin(); Ci!=sub_calculs.end(); Ci++){
			result = result + (Ci->tileList());
		}
	}
	return result;
}

//return the list of values from the sub-calculs of depth only 1
selection calcul::valuesList() const{
	selection result;
	list<calcul>::const_iterator Ci;
	for(Ci=sub_calculs.begin(); Ci!=sub_calculs.end(); Ci++){
		(result.tab[Ci->value])++;
	}
	return result;
	
}

//return the number of tiles used by a calcul, used by game::displayBase().
int calcul::length() const{
	if(isTile) return 1;
	
	int result=0;
	list<calcul>::const_iterator Ci;
	for(Ci=sub_calculs.begin(); Ci!=sub_calculs.end(); Ci++){
		result+=Ci->length();
	}
	return result;
}

//--------------------------------------------------------------
//Canonization functions
//--------------------------------------------------------------

//all canonizations are active by default
bool calcul::cano1_active=true;
bool calcul::cano2_active=true;
bool calcul::cano3_active=true;

//return false if the calcul is not "canonical", i.e not "sufficiently simple"
bool calcul::isCanonical() const{
	if(isTile) return true;
	if(cano1_active && !isCanonical_1()) return false; // 2+2 is not canonical but 2*2
	if(cano2_active && !isCanonical_2()) return false; // No way to obtain the same value with less tiles
	if(cano3_active && !isCanonical_3()) return false; // allow only factorized forms
	return true;
}

// 2+2 is not canonical but 2*2 is (easier to code because 2/2=1=usefull, 2-2=0=useless).
//return true if no sub-calcul of depth 1 in a broad meaning is of type 2+2 (comme 100-2-10/5)
//warning : slighlty more efficient than intended, and also consider as non canonical 2-2
//which is also considered by isCanonical_2
bool calcul::isCanonical_1() const{
	if(isAdditive){
		list<calcul> liste=sub_calculs_list();
		liste.insert(liste.end(), sub_calculs.begin(), sub_calculs.end());
		list<calcul>::iterator Ci,Cj,Cnext;
		for(Ci=liste.begin();Ci!=liste.end();Ci=Cnext){
			Cnext=Ci; Cnext++;
			if(Ci->value!=2){
				liste.erase(Ci);
			}
		}
		if(liste.size()>1){
			selection values_du_calcul=valuesList();
			for(Ci=liste.begin();Ci!=liste.end();Ci++){
				for(Cj=Ci;Cj!=liste.end();Cj++){
					if((Ci->valuesList() + Cj->valuesList()).isIncludedIn(values_du_calcul)){
						return false;
					}
				}
			}
		}
	}
	return true;
}

//No sub-calcul in a broad meaning should contain a subset of tiles that allow to reach the same target
bool calcul::isCanonical_2() const{
	list<targetType> liste=subSet_reachableTarget(false);
	list<targetType>::iterator Vi;
	
	for(Vi=liste.begin();Vi!=liste.end();Vi++){
		if(*Vi==value){
			return false;
		}
	}
	
	list<calcul> liste_sous_calc=sub_calculs_list();
	list<calcul>::iterator Ci;
	
	for(Ci=liste_sous_calc.begin();Ci!=liste_sous_calc.end();Ci++){
		list<targetType> liste2=Ci->subSet_reachableTarget(false);
		for(Vi=liste2.begin();Vi!=liste2.end();Vi++){
			if(*Vi==Ci->value){
				return false;
			}
		}
	}
	
	return true;
}

//a calcul is considered canonical if it is in a factorized form, ie 100*7+5*7 is not canonical.
bool calcul::isCanonical_3() const{
	if(isTile || (!isAdditive)){
		return true;		//factorization is possible only for additive calcul
	}
	list<calcul>::const_iterator Ci;
	list<targetType> LC;
	list<targetType>::const_iterator CPTj;
	set<targetType> factoCandidates;		// set of possible values of the factorization (those from the first sub_calcul)
	set<targetType>::iterator CPTi,CPTsuiv;
	
	//-----------if a sub_calcul is a tile, there is no possible factorization--------
	for(Ci=sub_calculs.begin();Ci!=sub_calculs.end();Ci++) {
		if(Ci->isTile){
			return true;
		}
	}
	
	//-------main algo------------------------------------------------
	Ci=sub_calculs.begin();
	LC=Ci->subTargetList();
	for(CPTj=LC.begin();CPTj!=LC.end();CPTj++){
		factoCandidates.insert(*CPTj);
	}
	
	for(Ci++;Ci!=sub_calculs.end();Ci++){
		LC=Ci->subTargetList();
		for(CPTi=factoCandidates.begin();CPTi!=factoCandidates.end();CPTi=CPTsuiv){
			CPTsuiv=CPTi; CPTsuiv++;
			bool appartient=false;
			for(CPTj=LC.begin();CPTj!=LC.end();CPTj++){
				if(*CPTj == *CPTi){
					appartient=true;
				}
			}
			if(!appartient){
				factoCandidates.erase(CPTi);
			}
		}
	}
	
	bool found=false;		//will be true if we find a factorization
	for(CPTi=factoCandidates.begin();CPTi!=factoCandidates.end();CPTi++){		//loop to avoid the problem of (7+7)x8-8/(1+1)
		if(*CPTi < 0){		//if the candidate divides, there is no problem
			found=true;
		}
		else{			//case where the candidate multiply
			bool correct=true;
			for(Ci=sub_calculs.begin();Ci!=sub_calculs.end();Ci++){
				if((Ci->value % (*CPTi))!=0){
					correct=false;
				}
			}
			if(correct){
				found=true;
			}
		}
	}
	
	//the calcul is canonical if we have not found a factorization
	return !found;
}
