/***************************************************************************
 *   Copyright (C) 2006-2012 by Julien Lemoine, Simon Viennot              *
 *   Web site: http://kitsune.tuxfamily.org                                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "game.h"

//QObject used only for the tr function (multi-language support)
#include <QObject>

//launch the computation as a separate thread
//this function is called through the function start()
void game::run() {
	launchAnalysis();
}

//initialize the game class with a given selection
void game::init(const selection& t, const int &liminf, const int &limsup){
	//given selection.
	givenSelection=t;
	
	//subSelections contains the list of subselection of the given selection, in a correct order
	subSelections=givenSelection.computeSubSelections();
	
	//base to store the calculs
	stock emptyStock;
	completeBase.clear();
	completeBase.assign(subSelections.size(), emptyStock);
	
	//initialize the limits of the range in which final solutions will be
	//these limits are improved all along the analysis
	lim_inf_inf=0;		//varies between 0 and 99
	lim_inf_sup=100000;	//varies between 100 and 100000
	lim_sup_inf=0;		//varies between 0 and 999
	lim_sup_sup=100000;	//varies between 1000 and 100000
	
	lim_inf=liminf;
	lim_sup=limsup;
}

//analyze the given selection
//the game class must be initialized before calling this function
void game::launchAnalysis(){
	int subSize=subSelections.size();
	
	int i;
	stock::iterator Cj, Ck, Ck_first;
	
	//main loop
	for(currentStock=1; currentStock<subSize; currentStock++){
		last_subSelection=(currentStock==subSize-1);	//indicate if it is the last subSelection
		
		if((subSelections[currentStock]).size()==1){		//case of a subSelection with only one tile
			map<tile, int>::iterator it=subSelections[currentStock].tab.begin();
			completeBase[currentStock].insert(calcul(it->first));
		}
		else{	//case of a subSelection with at least 2 tiles
			vector<selection> sub_subSelection=(subSelections[currentStock]).computeSubSelections();
			int sous_sous_taille=sub_subSelection.size();
			
			//loop : in the case of the subselection 2;3;5, we need to use 2 + 3;5 et 3 + 5;2 etc
			for(i=1; i<(sous_sous_taille+1)/2; i++) {
				int position1=(sub_subSelection[i]).find_inside(subSelections);
				int position2=(sub_subSelection[sous_sous_taille-1-i]).find_inside(subSelections);
				
				for (Cj=completeBase[position1].begin(); Cj!=completeBase[position1].end(); Cj++) {
					if (position1!=position2) {
						Ck_first=completeBase[position2].begin();
					} 
					else {
						Ck_first=Cj;
					}
					
					for (Ck=Ck_first; Ck!=completeBase[position2].end(); Ck++) {
						combinations(*Cj, *Ck);
					}
				}
			}
		}
		
		deleteNonCano();	//delete non-canonical calculs from the current stock
	}
	
	//delete the calculs out of the final range
	deleteOutOfRange();
}

//return true if the calcul is in the correct range
//(between 100 and 999, and maybe <100 but >=lim_inf_inf, and the same for >999)
//this function changes lim_x_x variables when some calcul enables to refine them
bool game::inTheRange(const calcul& C){
	targetType value=C.value;
	if(value>=lim_inf && value<=lim_sup){		// case 100<=value<=999
		if(lim_inf_sup>value){
			lim_inf_sup=value;
		}
		if(lim_sup_inf<value){
			lim_sup_inf=value;
		}
		return true;
	}
	else{
		if(value<lim_inf){				// case value<100
			if(lim_inf_inf<=value){		//we need = or some calculs could be considered wrongly out of the range
				lim_inf_inf=value;
				return true;
			}
		}
		else{						// case value>999
			if(lim_sup_sup>=value){
				lim_sup_sup=value;
				return true;
			}
		}
	}
	return false;
}

//add a calcul in the base if it is in the range and not the last subSelection
void game::addToBase(const calcul& A){
	if(!last_subSelection || inTheRange(A)){
		completeBase[currentStock].insert(A);
	}
}

//compute the combinations of A and B, and add it to the base
void game::combinations(const calcul& A, const calcul& B){
	addToBase(A+B);						//take addition into account
	
	if(A.value<B.value) {				//take soustraction into account
		addToBase(B-A);
	}
	else if(A.value>B.value) {
		addToBase(A-B);
	}
	
	if(A.value!=1 && B.value!=1) {			//take multiplication into account
		addToBase(A*B);
				
		if(A.value==B.value) {			//take division into account
			calcul C=A/B; calcul D=B/A;
			if(C < D) {
				addToBase(C);
			} else {
				addToBase(D);
			}
		}
		else {
			if (A.value % B.value==0){
				addToBase(A/B);
			}
			else if (B.value % A.value==0){
				addToBase(B/A);
			}
		}
	}
}

//delete non canonical positions from the current stock
void game::deleteNonCano(){
	stock::iterator Ci, Cnext;
	
	for(Ci=completeBase[currentStock].begin(); Ci!=completeBase[currentStock].end(); Ci=Cnext) {
		Cnext=Ci; Cnext++;
		if(! (Ci->isCanonical())){
			completeBase[currentStock].erase(Ci);
		}
	}
}

//delete from the base the positions "out of the range"
void game::deleteOutOfRange() {
	int i;
	int subSize=subSelections.size();
	stock::iterator Cj;
	
	//loop to find the range in which the solutions need to be (lim_inf_inf et lim_sup_sup)
	for(i=1;i<subSize-1;i++){
		for(Cj=completeBase[i].begin(); Cj!=completeBase[i].end(); Cj++){
			inTheRange(*Cj);
		}
	}
	
	//take into account the follwing case : if we want to reach the target 100, and we can reach 101,
	//then it is useless to keep 98, which is more distant
	//Find the final lim_x_x
	if(lim_inf-lim_inf_inf>lim_inf_sup-lim_inf){
		lim_inf_inf=lim_inf;
	}
	
	if(lim_sup_sup-lim_sup>lim_sup-lim_sup_inf){
		lim_sup_sup=lim_sup;
	}
	
	//loop to delete the calculs which are not between the final lim_inf_inf and lim_sup_sup
	stock::iterator Cnext;
	
	for(i=1;i<subSize;i++){
		for(Cj=completeBase[i].begin();Cj!=completeBase[i].end();Cj=Cnext){
			Cnext=Cj; Cnext++;
			if(Cj->value < lim_inf_inf || Cj->value > lim_sup_sup){
				completeBase[i].erase(Cj);
			}
		}
	}
}

//return the distance to the solution : 0 if the problem is solved, or the distance between the target and the best solution
//this function is usefull only if we analyze only one target, ie if lim_inf=lim_sup
int game::distanceToSolution() const{
	int result=1000;
	vector<stock>::const_iterator STi;
	stock::const_iterator Cj;
	for(STi=completeBase.begin(); STi!=completeBase.end(); STi++) {
		for(Cj=STi->begin(); Cj!=STi->end(); Cj++) {
			if((Cj->value)>=lim_inf){
				if((Cj->value -lim_inf)<result){
					result=(Cj->value -lim_inf);
				}
			}
			else{
				if((lim_inf- Cj->value)<result){
					result=(lim_inf- Cj->value);
				}
			}
		}
	}
	return result;
}

//display the solutions with priority to solutions with less tiles
//can be used only in the case of studying a unique target
//if the target is not reachable, display the best approximations
//note : tr has not effect when Qt is not active
QString game::displayBase() const {
	QString result;
	string str_temp;

	int i;
	int dist=distanceToSolution();
	stock stock_temp;
	vector<stock>::const_iterator STi;
	stock::const_iterator it;
	for(i=1;i<=givenSelection.size();i++){
		stock_temp.clear();
		for(STi=completeBase.begin(); STi!=completeBase.end(); STi++) {
			for(it=STi->begin(); it!=STi->end(); it++) {
				if(it->length()==i && ((it->value -lim_inf)==dist || (lim_inf- it->value)==dist)){
					stock_temp.insert(*it);
				}
			}
		}
		if(!stock_temp.empty()){
			if(distanceToSolution()==0) {
				result.append(QObject::tr("Solutions with %1 tiles:").arg(i));
			} else {
				result.append(QObject::tr("Best approximations with %1 tiles :").arg(i));
			}
			result.append("\n");
			for(it=stock_temp.begin(); it!=stock_temp.end(); it++) {
				stringstream ss2;
				ss2 << it->value;
				ss2 >> str_temp;
				result.append(QString::fromStdString(str_temp));
				result.append(".  ");
				result.append(QString::fromStdString(it->display()));
				result.append("\n");
			}
			result.append("\n");
		}
	}
	return result;
}

QString game::displayProblem() const {
	QString result;
	
	//selection
	result.append(QString(tr("Selection : ")));
	
	//display the selection in the tmp string
	string tmp;
	stringstream flux;
	flux << givenSelection;
	flux >> tmp;
	
	result.append(QString(tmp.c_str()));
	result.append(QString("\n"));
	
	//target
	result.append(QString(tr("Target : ")));
	result.append(QString("%1").arg(lim_inf));
	result.append(QString("\n\n"));
	
	return result;
}

//vector for all the possible targets, 1 if the target is reached, or 0 if not
vector<bool> game::reachedTarget() const{
	vector<bool> result;
	result.assign(1000,false);
	vector<stock>::const_iterator STi;
	stock::iterator Cj;
	
	for(STi=completeBase.begin(); STi!=completeBase.end(); STi++) {
		for(Cj=STi->begin();Cj!=STi->end();Cj++){
			if(Cj->value>=lim_inf && Cj->value<=lim_sup){
				result[Cj->value]=true;
			}
		}
	}
	
	return result;
}
