/***************************************************************************
 *   Copyright (C) 2006-2012 by Julien Lemoine, Simon Viennot              *
 *   Web site: http://kitsune.tuxfamily.org                                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "playerwidget.h"
#include "optionswidget.h"

#include <QtGui>

void PlayerWidget::addWin() {
	totalWin++;
	saveWinLoss();
	refresh();
}

void PlayerWidget::addLoss() {
	totalLoss++;
	saveWinLoss();
	refresh();
}

void PlayerWidget::readWinLoss() {
	//read the total number of win and loss in the options file
	//caution : this code works only if there is only one player
	if(OptionsWidget::saveOptions) {
		QString optionsFilePath=QDir::homePath()+ QString("/.kitsune.options.") + global::kitsuneVersion;
		QSettings settings(optionsFilePath, QSettings::IniFormat);
		totalWin=settings.value("totalWin").toInt();
		totalLoss=settings.value("totalLoss").toInt();
		scoreDate=settings.value("scoreDate").toString();
	} else {
		totalWin=0;
		totalLoss=0;
	}
	refresh();
}

void PlayerWidget::saveWinLoss() {
	//save the total number of win and loss in the options file
	//caution : this code works only if there is only one player
	if(OptionsWidget::saveOptions) {
		QString optionsFilePath=QDir::homePath()+ QString("/.kitsune.options.") + global::kitsuneVersion;
		QSettings settings(optionsFilePath, QSettings::IniFormat);
		settings.setValue("totalWin", totalWin);
		settings.setValue("totalLoss", totalLoss);
		if(scoreDate.isEmpty()){
			//there was no previous record: record current date
			settings.setValue("scoreDate", QDate::currentDate().toString(Qt::ISODate));
		}
	}
}

PlayerWidget::PlayerWidget( QWidget *parent ) : QWidget( parent) {
	QHBoxLayout *hbox_Layout = new QHBoxLayout( this );
	hbox_Layout->setSpacing(0);
	hbox_Layout->setMargin(0);
	
	QWidget *vbox = new QWidget( this );
	QVBoxLayout *vbox_Layout = new QVBoxLayout( vbox );
	vbox_Layout->setSpacing(0);
	vbox_Layout->setMargin(0);
	
	//Display number of loss/win
	winLabel = new QLabel;
	winLabel->setFont( font_normal );
	lossLabel = new QLabel;
	lossLabel->setFont( font_normal );
	
	vbox_Layout->addWidget(winLabel);
	vbox_Layout->addWidget(lossLabel);
	
	hbox_Layout->addStretch( 1 );
	hbox_Layout->addWidget(vbox);
	hbox_Layout->addStretch( 1 );
	
	totalWin=0;
	totalLoss=0;
}

//refresh the labels displaying the number of wins and losses
void PlayerWidget::refresh() {
	QString s = QString( tr( "Win  : %1 " )).arg( totalWin, 0, 10 );
	winLabel->setText(s);
	
	s = QString( tr( "Loss : %1 " )).arg( totalLoss, 0, 10 );
	lossLabel->setText(s);
}

//called when we click with the mouse on the player widget
//we call the score dialog (popup with score details)
void PlayerWidget::mousePressEvent(QMouseEvent *event) {
	event->accept();
	displayScoreDialog();
}

//display a popup with the score details
//called by a mouse click on the player widget, or from Kitsune menu
void PlayerWidget::displayScoreDialog() {
	selectDialog = new QDialog(this);
	selectDialog->setWindowTitle(QString(tr("Score details")));
	QVBoxLayout *vlayout = new QVBoxLayout;
	
	//display date
	QString dateIntro = QString(tr("Score since : "));
	
	QLabel *dateLabel = new QLabel;
	dateLabel->setFont( font_normal );
	if(scoreDate.isEmpty()) {
		dateLabel->setText(dateIntro + QDate::currentDate().toString(Qt::ISODate));
	} else {
		dateLabel->setText(dateIntro + scoreDate);
	}
	vlayout->addWidget(dateLabel);
	
	//Display number of loss/win
	QLabel *win = new QLabel;
	win->setFont( font_normal );
	QLabel *loss = new QLabel;
	loss->setFont( font_normal );
	QString s = QString( tr( "Win  : %1 " )).arg( totalWin, 0, 10 );
	win->setText(s);
	s = QString( tr( "Loss : %1 " )).arg( totalLoss, 0, 10 );
	loss->setText(s);
	
	vlayout->addWidget(win);
	vlayout->addWidget(loss);
	
	//reset score button
	QPushButton *pbReset = new QPushButton( tr( "Reset score" ) );
	connect(pbReset, SIGNAL(clicked()), this, SLOT(resetScore()) );
	vlayout->addWidget(pbReset);
	
	//"OK" button to close window
	QPushButton *pbOk = new QPushButton( tr("OK"));
	pbOk->setFont( font_normal );
	pbOk->setFocus();
	connect( pbOk, SIGNAL( clicked() ), selectDialog, SLOT( accept() ) );
	vlayout->addWidget(pbOk);
	
	selectDialog->setMinimumSize(200, 100);
	selectDialog->setLayout(vlayout);
	selectDialog->resize(selectDialog->sizeHint());
	selectDialog->exec();
}

void PlayerWidget::resetScore() {
	int ret=QMessageBox::information (this, tr("Reset score"), 
			 tr("Do you really want to reset the score ?"), QMessageBox::Ok|QMessageBox::Cancel);
	
	if(ret!=QMessageBox::Ok) return;
	
	totalWin=0;
	totalLoss=0;
	scoreDate="";
	saveWinLoss();
	selectDialog->accept();
	refresh();
}
