/***************************************************************************
 *   Copyright (C) 2006-2012 by Julien Lemoine, Simon Viennot              *
 *   Web site: http://kitsune.tuxfamily.org                                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "problem.h"

#include <QtGui>
#include <QtXml>

//note : this function is not used in version 3.0
//test will be needed before use in the next version
void Problem::writeXmlFile(QString fileName) {
	if(fileName == QString(""))
		return;

	QFile f(fileName);
	if ( !f.open( QIODevice::WriteOnly ) ) {
		QMessageBox::critical( 0, tr("Error"), tr("Can't write the file."));
		return;
	}
	
	QTextStream text( &f);

	QDomDocument doc("Kitsune");
	QDomElement root = doc.createElement("Kitsune");
	root.setAttribute("version", "3.0");	
	if(!commentBefore.isEmpty()){
		root.setAttribute("commentBefore", commentBefore);
	}
	if(!commentAfter.isEmpty()){
		root.setAttribute("commentAfter", commentAfter);
	}
	root.setAttribute("winPoint", winPoint);
	root.setAttribute("reachedPoint", reachedPoint);
	
	doc.appendChild(root);

	vector<Param>::const_iterator Vi;

	for(Vi=problemList.begin(); Vi!=problemList.end(); Vi++) {
		QDomElement p = doc.createElement("problem");
		p.setAttribute("target", (qlonglong) (Vi->target) );
		p.setAttribute("time", Vi->roundTime );
		if(!Vi->commentBefore.isEmpty()){
			p.setAttribute("commentBefore", Vi->commentBefore );
		}
		if(!Vi->commentAfter.isEmpty()) {
			p.setAttribute("commentAfter", Vi->commentAfter );
		}
		root.appendChild(p);
		
		vector<tile>::const_iterator Tj;
		for(Tj=Vi->tileList.begin(); Tj!=Vi->tileList.end(); Tj++) {
			QDomElement q = doc.createElement("tile");
			q.setAttribute("value", (int) (*Tj) );
			p.appendChild(q);
		}
	}

	text << doc.toString();

	f.close();
}

void Problem::readXmlFile(QString fileName) {
	if(fileName == QString(""))
		return;

	QFile file(fileName);
	if ( !file.open( QIODevice::ReadOnly ) ) {
		QMessageBox::critical( 0, tr("Error"), tr("Can't read the file."));
		return;
	}
	
	QDomDocument doc("Kitsune");

	QString errMsg;
	int errLine, errColumn;
	if ( !doc.setContent( &file, false, &errMsg, &errLine, &errColumn ) ) {
		QMessageBox::critical( 0, tr("Error"), tr("This is not a Kitsune file or the format is wrong.\n%1 at line %2 and column %3.").arg(errMsg, errLine, errColumn));
		file.close();
		return;
	}
	file.close();

	//parse the xml tree
	problemList.clear();
	QDomElement root = doc.documentElement();

	QString version = root.attribute("version");
	if(version != QString("3.0")) {
		QMessageBox::critical( 0, tr("Error"), tr("The file version %1 is unknown. \nIt probably means you need to upgrade to a new version of Kitsune.").arg(version));
		return;	
	}
	
	commentBefore = root.attribute("commentBefore");
	commentAfter = root.attribute("commentAfter");
	winPoint = root.attribute("winPoint").toInt();
	reachedPoint = root.attribute("reachedPoint").toInt();
	author = root.attribute("author");
	date = root.attribute("date");
	
	QDomNode problemNode = root.firstChild();
	while( !problemNode.isNull() ) {
		if ( problemNode.isElement() && problemNode.nodeName() == "problem" ) {
			//this is a new problem node
			Param prob;
			QDomElement p = problemNode.toElement();
			
			//target (must be in the range [1-999])
			prob.target = p.attribute("target").toInt();
			if(prob.target<1 || prob.target>999) {
				prob.target = 100;
				prob.debugMessage = QString(tr("Target out of [1-999] is replaced by 100."));
			}
			
			//given time (must be at least than 10s)
			prob.roundTime = p.attribute("time").toInt();
			if(prob.roundTime<10) {
				prob.roundTime = 10;
				prob.debugMessage = QString(tr("Given time less than 10 seconds is replaced by 10 seconds."));
			}
			
			prob.commentBefore = p.attribute("commentBefore");
			prob.commentAfter = p.attribute("commentAfter");
			
			//read the list of tiles
			QDomNode tileNode = problemNode.firstChild();
			while ( !tileNode.isNull() ) {
				if ( tileNode.isElement() && tileNode.nodeName() == "tile" ) {
					int v = tileNode.toElement().attribute("value").toInt();
					
					//add the tile to the list only if there is less than six tiles
					if(prob.tileList.size()<6) {
						if(v<1 || v>255) {
							//a tile out of [1-255] is invalid and replaced by 1
							v=1;
							prob.debugMessage = QString(tr("Tiles out of [1-255] are replaced by 1."));
						}
						
						prob.tileList.push_back((tile) v);
					} else {
						prob.debugMessage = QString(tr("Too many tiles. Only 6 tiles can be used."));
					}
				}
				tileNode = tileNode.nextSibling();
			}
			
			int nbTiles = prob.tileList.size();
			if(nbTiles<3) {
				prob.debugMessage = QString(tr("Too less tiles. At least 3 tiles are needed. Missing tiles are replaced by 1."));
				
				while(nbTiles<3) {
					prob.tileList.push_back(1);
					nbTiles++;
				}
			}
			
			problemList.push_back(prob);
		}
		problemNode = problemNode.nextSibling();
	}
}

QString Problem::intro() {
	QString result;
	result += commentBefore;
	result += QString("\n\n");
	result += QString(tr("%1 problems").arg(problemList.size()));
	result += QString("\n");
	
	//display the points only if they are defined to values different from zero
	if(winPoint!=0 || reachedPoint!=0) {
		result += QString(tr("Solved in time -> %1 points\nSolved out of time -> %2 points").arg(winPoint).arg(reachedPoint));
	}
	
	result += QString("\n\n");
	if(!author.isEmpty()) {
		result += QString(tr("Author : %1").arg(author));
		result += QString("\n");
	}
	if(!date.isEmpty()) {
		result += QString(tr("Date : %1").arg(date));
	}
	
	return result;
}

QString Problem::scoreResult() {
	QString result;	
	if(!commentAfter.isEmpty()){
		result = commentAfter + "\n\n";
	}
	
	result += QString(tr("Score details : ")) + "\n\n";
	
	//display the list of problems and results achieved by the player
	vector<Param>::iterator P;
	vector<tile>::iterator t;

	int nb_win=0, nb_solved=0, nb_loss=0;
	
	for(P = problemList.begin(); P!= problemList.end(); P++) {		
		//selection
		result += QString(tr("Selection : "));
		for(t=P->tileList.begin(); t != P->tileList.end(); t++) {
			result += QString("%1 ").arg( (int) (*t) );
		}
		result += QString("\n");
		
		//target
		result += QString(tr("Target : ")) + QString("%1").arg(P->target) + QString("\n");
		
		//result achieved by the player
		if(P->victory) {
			result +=QString(tr("Win"));
			nb_win++;
		} else
		if(P->reached) {
			result += QString(tr("Solved"));
			nb_solved++;
		} else {
			result += QString(tr("Loss"));
			nb_loss++;
		}
		result += QString("\n\n");
	}
	
	//total score
	bool displayScore=false;
	if(winPoint!=0 || reachedPoint!=0) displayScore=true;
	
	QString totalScore;
	if(nb_win>0) {
		QString points;
		if(displayScore) {
			points=QString(" -> ") + QString(tr("%2 points").arg(nb_win*winPoint));
		}
		
		totalScore += QString(tr("Win  : %1").arg(nb_win)) + points;
		totalScore += QString("\n");
	}
	if(nb_solved>0) {
		QString points;
		if(displayScore) {
			points=QString(" -> ") + QString(tr("%2 points").arg(nb_solved*reachedPoint));
		}
		
		totalScore += QString(tr("Solved  : %1").arg(nb_solved)) + points;
		totalScore += QString("\n");
	}
	if(nb_loss>0) {
		totalScore += QString(tr("Loss  : %1").arg(nb_loss));
		totalScore += QString("\n\n");
	}
	
	if(displayScore) {
		totalScore += QString(tr("Total score: %1 / %2").arg(nb_win*winPoint+nb_solved*reachedPoint).arg(problemList.size()*winPoint) );
		totalScore += QString("\n\n");
	}
	
	result.insert(0, totalScore);
	
	return result;
}

