/***************************************************************************
 *   Copyright (C) 2006-2012 by Julien Lemoine, Simon Viennot              *
 *   Web site: http://kitsune.tuxfamily.org                                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "selection.h"

int selection::nb_tiles=6;
int selection::minimalTarget=100;
int selection::maximalTarget=999;
bool selection::possible_target=true;
map<tile, int> selection::ref;

map<tile, int> selection::getDefaultRules() {
	map<tile, int> rules;
	
	for(int i=1; i<=10; i++) {
		rules[i]=2;
	}
	rules[25]=1;
	rules[50]=1;
	rules[75]=1;
	rules[100]=1;
	
	return rules;
}

//return the size of a selection
int selection::size() const {
	int result=0;
	map<tile, int>::const_iterator it;
	
	for(it=tab.begin();it!=tab.end();it++){
		result=result+(it->second);
	}
	
	return result;
}

//init tab with authorized tiles
void selection::initWithPossibleTiles() {
	tab.clear();
	int i;
	tile firstTile = ref.begin()->first; //get the first possible tile allowed by the rules
	tile currentTile = firstTile;
	for(i=1;i<=selection::nb_tiles;i++){
		tab[currentTile]++;
		currentTile = leastTileSup(*this, currentTile); //next available tile superior to the current one
		if(currentTile==0) currentTile = firstTile; //if there are less than 6 different tiles
	}
}

vector<tile> selection::getTileList() {
	vector<tile> result;
	map<tile, int>::iterator it;
	
	for(it=tab.begin();it!=tab.end();it++){
		int j;
		for(j=0; j< (it->second); j++) {
			result.push_back(it->first);
		}
	}
	
	return result;
}

//--------------------------------------------------------------
//Operations with selections
//--------------------------------------------------------------
bool selection::operator!=(const selection& a) const{
	return (tab!=a.tab);
}

bool selection::operator==(const selection& a) const {
	return (tab==a.tab);
}

bool selection::operator<(const selection& a) const {
	return (tab<a.tab);
}

selection selection::operator+(const selection& a) const {
	selection result=a;
	map<tile, int>::const_iterator it;
	for(it=tab.begin();it!=tab.end();it++){
		result.tab[it->first] += (it->second);
	}
	return result;
}

//inclusion operator in a broad meaning
//for example 1;1;4 is included in 1;1;3;4;100 but not in 1;4;10;50
bool selection::isIncludedIn(selection a) const {
	map<tile, int>::const_iterator it;
	for(it=tab.begin();it!=tab.end();it++){
		if ((it->second) > a.tab[it->first]){
			return false;
		}
	}
	return true;
}

//compute the sub-selection sorted in a "correct" order
vector<selection> selection::computeSubSelections() const {
	selection emptySelection;
	
	//vector with only an empty selection
	vector<selection> v(1,emptySelection);
	
	map<tile, int>::const_iterator i;
	for(i=tab.begin();i!=tab.end();i++){	//TODO : detail the algorithm
		int a=v.size();		//caution : the size of v changes all along the loop indexed by j
		int j; int k;
		for(j=1;j<=i->second;j++){
			for(k=0;k<a;k++){
				selection temp=v[k];
				temp.tab[i->first]=j;
				v.push_back(temp);
			}
		}
	}
	
	return v;
}

//find the position of a selection in a list of selections
int selection::find_inside(const vector<selection> &v) const{
	int result=0;
	while (v[result]!=*this) result++;
	return result;
}

//display the selection on a given stream (used with cout)
ostream& operator<<(ostream& ostr, const selection& t) {
	if (t.tab.size()==0) {
		return ostr << "empty";
	}
	
	map<tile, int>::const_iterator i; int j;
	for(i=t.tab.begin(); i!=t.tab.end(); i++){
		for(j=0; j<(i->second); j++){
			ostr << (int) (i->first) << ".";
		}
	}
	
	return ostr;
}

//return the smallest tile superior to a given tile p, and available when taking into account the selection t
//return "0" if no tiles are available
tile leastTileSup(selection t,tile p) {
	tile result=incrementTile(p);
	if(result==0) return result;
	while (t.tab[result]==selection::ref[result]){
		result=incrementTile(result);
		if(result==0) return result; //this is needed because if we do the test in the while with 0, 0 is added to the map
	}
	return result;
}

//return the smallest tile inferior to a given tile p, and available when taking into account the selection t
//return "0" if no tiles are available
tile leastTileInf(selection t,tile p) {
	tile result=decrementTile(p);
	if(result==0) return result;
	while ( t.tab[result]==selection::ref[result]) {
		result=decrementTile(result);
		if(result==0) return result; //this is needed because if we do the test in the while with 0, 0 is added to the map
	}
	return result;
}

//return the next tile bigger than the given tile p, or "0" if there is not
tile incrementTile(tile p) {
	map<tile, int>::iterator it;
	it=selection::ref.find(p);
	if(it!=selection::ref.end()) it++; // next tile
	
	if(it!=selection::ref.end()) {
		return it->first;
	} else	{
		return 0;
	}
}

//return the previous tile smaller than the given tile p, or "0" if there is not
tile decrementTile(tile p) {
	map<tile, int>::iterator it;
	it=selection::ref.find(p);
	if(it!=selection::ref.end()) it--; //previous tile
	
	if(it!=selection::ref.end()) {	//in case of an empty list
		return it->first;
	} else	{
		return 0;
	}
}
